function varargout=GeneralizedNewtonFractalGPUsimple(z0,varargin);
% GeneralizedNewtonFractalGPU solves the nonlinear equation z^3-1=0
%   Z=GeneralizedNewtonFractalGPUsimple(z0) 
%   solves the nonlinear equations z^3-1=0 using 
%   Newtons methods with z real or complex, starting with 
%   the initial guess z0 up to a default tolerance 1e-8, 
%   doing at most 200 iterations. 
  import Thrust.samples.NewtonFractal.*;
  p = inputParser;
  p.addParamValue('tol', 1e-8, @isscalar );
  p.addParamValue('itermax', 200, @isscalar );
  p.addParamValue('verbose', 0, @(x) ismember(x,0:1) );
  p.addParamValue('a', 1, @isscalar );
  p.parse(varargin{:});
  R=p.Results;
  
  if R.verbose,fprintf('1) Transfert/init datas on GPU\n');end
  tstart=tic();
  gz0=mThrust(z0(:),'device'); % Only vectors are supported on mThrust
  gz=mThrust(numel(z0),'device','double',true);
  gcount=mThrust(numel(z0),'device','unsigned',false);
  cuda.DeviceSynchronize();
  T(1)=toc(tstart);
  if R.verbose,fprintf('2) Computation on GPU\n');end
  tstart=tic();
  mex_GeneralizedNewtonFractal_simple(complex(R.a), gz0, uint32(R.itermax), R.tol, gz, gcount);
  cuda.DeviceSynchronize();
  T(2)=toc(tstart);
  if R.verbose,fprintf('3) Transfert datas on Matlab\n');end
  tstart = tic();
  z=reshape(gather(gz),size(z0));
  count=reshape(gather(gcount),size(z0));
  cuda.DeviceSynchronize();
  T(3) = toc( tstart ); % transfert time to CPU
  if R.verbose
    fprintf('   to GPU : %.3f(s), computation: %.3f(s),',T(1:2))
    fprintf(' to CPU: %.3f(s)\n',T(3))
    fprintf('  -> total: %.3f(s)\n',sum(T));
  end
  if nargout>=1,varargout{1}=z;end
  if nargout>=2,varargout{2}=count;end
  if nargout==3,varargout{3}=T;end
end