clear all
close all
import Thrust.samples.NewtonFractal.*
% Use -singleCompThread option when starting MATLAB to limit MATLAB to a single computational thread

f=@(z) z.^3-1;
df=@(z) 3*z.^2;
sf='z*z*z-1.00'; % Thrust function in z
sdf='3.0*z*z';
tol=1e-10;
itermax=300;
a=1;
box=[-2,2,-2,2];


fprintf('0) CUDA/Mex compilation : \n')
P=Thrust.tools.getCurrentFunctionPath();
tstart=tic();
file=create_funcs(sf,sdf,'file','funcs.cuh','verbose',0);
cudamexfile=[P,'mex_GeneralizedNewtonFractal_full.cu'];
dep=[P,'funcs.cuh'];
Thrust.compile.mex(cudamexfile,'samepath',true,'verbose',0,'dep',dep)
Tcomp=toc(tstart);
fprintf('  -> Compilation in %.3f(s)\n',Tcomp);
tstart=tic();
GeneralizedNewtonFractalGPU(sf,sdf,complex(1,1),'compile',false);
Tcomp(2)=toc(tstart);
fprintf('  -> Preload mex file in %.3f(s)\n',Tcomp(2));



LN=1000:500:5000;LN=[LN;LN]';
%LN=1000:500:2000;LN=[LN;LN]';
n=size(LN,1);
Tcpu=zeros(n,1);
Tgpu=zeros(n,3);
fprintf('----------------------------------------\n')
fprintf('Bench of generalized Newton fractals\n')
fprintf('  base : Matlab not vectorized version\n')
fprintf('  vec  : Matlab vectorized version\n')
fprintf('  gpu  : GPU version\n')
fprintf('----------------------------------------\n')
fprintf('  f=%s, f''=%s  (matlab datas)\n', ...
        Thrust.tools.strfunhandle(f),Thrust.tools.strfunhandle(df));
fprintf('  f=%s, f''=%s  (CUDA/Thrust datas)\n',sf,sdf);
fprintf('  tol=%g, itermax=%g, box=[%g,%g,%g,%g]\n',tol,itermax,box);
fprintf('----------------------------------------\n')
fprintf('1) Starting benchmark : \n')
for i=1:n
  fprintf(' (%d/%d) on %d-by-%d grid\n',i,n,LN(i,:));
  z0=complexGrid(box,LN(i,:));
  [~,~,T]=GeneralizedNewtonFractalVec(f,df,z0,'tol',tol, 'itermax',itermax,'a',a);
  Tcpu(i)=T;
  fprintf(' vec: %.3f(s),',T);
  [~,~,T]=GeneralizedNewtonFractalGPU(sf,sdf,z0,'tol',tol, 'itermax',itermax,'a',a,'compile',false);
  Tgpu(i,:)=T(2:4);
  fprintf(' gpu: %.3f(s)\n',sum(T(2:4)));
  fprintf(' gpu details: CPU->GPU %.3f(s), GPU computation: %.3f(s), GPU->CPU: %.3f(s)\n',T(2:4));
end
%    
%  DataFormat={'$%d\\times %d$','$%.3f$(s)', ...
%              '\\begin{tabular}{c} $%.3f$(s)\\\\ $\\times$ $%.2f$ \\end{tabular}'};
%  ColumnFormat='|r||*{2}{c|}';
%  RowFormat='\hline';
%  RowHeaderFormat='\hline \hline';
%  Header={'Grid','Matlab Vectorized','CUDA/Thrust'};
%  sTgpu=sum(Tgpu,2);
%  Data=[LN,Tcpu,sTgpu,Tcpu./sTgpu];
%  LaTeXFilename=[P,'toto.tex'];
%  Caption='essai';
%  Thrust.extern.LatexTabular.PrintLatexTabularArrayV2(Data,Header,Caption,DataFormat,ColumnFormat,RowFormat,RowHeaderFormat,LaTeXFilename)
