clear all
close all
import Thrust.samples.NewtonFractal.*

f=@(z) z.^3-1;
df=@(z) 3*z.^2;
sf='z*z*z-1.0'; % Thrust function in z
sdf='3.0*z*z';
tol=1e-10;
itermax=300;
a=1;
box=[-2,2,-2,2];


fprintf('0) CUDA/Mex compilation : \n')
P=Thrust.tools.getCurrentFunctionPath();
tstart=tic();
file=create_funcs(sf,sdf,'file','funcs.cuh','verbose',0);
cudamexfile=[P,'mex_GeneralizedNewtonFractal_full.cu'];
dep=[P,'funcs.cuh'];
Thrust.compile.mex(cudamexfile,'samepath',true,'verbose',0,'dep',dep)
Tcomp=toc(tstart);
fprintf('  -> Compilation in %.3f(s)\n',Tcomp);

LN=100:100:400;LN=[LN;LN]';
n=size(LN,1);
Tcpu=zeros(2,n);
Tgpu=zeros(3,n);
TmThrust=zeros(3,n);
fprintf('----------------------------------------\n')
fprintf('Bench of generalized Newton fractals\n')
fprintf('  base    : Matlab not vectorized version\n')
fprintf('  vec     : Matlab vectorized version\n')
fprintf('  mThrust : mThrust version (with GPU device)\n')
fprintf('  gpu     : GPU version\n')
fprintf('----------------------------------------\n')
fprintf('  f=%s, f''=%s  (matlab datas)\n', ...
        Thrust.tools.strfunhandle(f),Thrust.tools.strfunhandle(df));
fprintf('  f=%s, f''=%s  (CUDA/Thrust datas)\n',sf,sdf);
fprintf('  tol=%g, itermax=%g, box=[%g,%g,%g,%g]\n',tol,itermax,box);
fprintf('----------------------------------------\n')
fprintf('1) Starting benchmark : \n')
for i=1:n
  fprintf(' (%d/%d) on %d-by-%d grid\n',i,n,LN(i,:));
  z0=complexGrid(box,LN(i,:));
  [~,~,T]=GeneralizedNewtonFractal(f,df,z0,'tol',tol, 'itermax',itermax,'a',a);
  Tcpu(1,i)=T;
  fprintf(' base: %.3f(s),',T);
  [~,~,T]=GeneralizedNewtonFractalVec(f,df,z0,'tol',tol, 'itermax',itermax,'a',a);
  Tcpu(2,i)=T;
  fprintf(' vec: %.3f(s),',T);
  [~,~,T]=GeneralizedNewtonFractalGPU(sf,sdf,z0,'tol',tol, 'itermax',itermax,'a',a,'compile',false);
  Tgpu(:,i)=T(2:4);
  fprintf(' gpu: %.3f(s)\n',sum(T(2:4)));
  fprintf(' gpu details: CPU->GPU %.3f(s), GPU computation: %.3f(s), GPU->CPU: %.3f(s)\n',T(2:4));
  [~,~,T]=GeneralizedNewtonFractalmThrust(f,df,z0,'tol',tol, 'itermax',itermax,'a',a);
  TmThrust(:,i)=T;
  fprintf(' mThrust : %.3f(s)\n',sum(T));
  fprintf(' mThrust details: CPU->GPU %.3f(s), GPU computation: %.3f(s), GPU->CPU: %.3f(s)\n',T);
end
  
