function varargout=constructor(varargin)
  p = inputParser; 
  p.addParamValue('verbose', 0, @(x) isscalar(x) & isnumeric(x));
  p.parse(varargin{:});
  verbose=p.Results.verbose;
  if verbose>1, fprintf('TEST : mThrust constructor\n'); end
  res=test1(verbose);
  res=res+test2(verbose);
  res=res+test3(verbose);
  Thrust.valid.tools.printFinalRes('mThrust',res,verbose)
  if nargout==1, varargout{1}=res;end
end

function res=test1(verbose)
  if verbose>1, fprintf(' test 1 : constructor with 4 arguments\n'); end
  T=Thrust.types();
  C=Thrust.containers();
  fail=0;count=0;
  for iscomplex={ false , true }
    for type=T
      for container=C
  %      whos
  %        fprintf('%s %s',type{1},container{1});
        fail=fail+test1_simple(container{1},type{1},iscomplex{1},verbose);
        count=count+1;
      end
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 1 : [%d/%d] OK\n',count-fail,count); end
end

function failure=test1_simple(container,type,iscomplex,verbose)  
  failure=0;
  dim=randi(10^5);
  startline='  ';
  cmd=sprintf('x=mThrust(%d,''%s'',''%s'',%s);',dim,container,type,Thrust.tools.logical2str(iscomplex));
  if verbose>2,fprintf('%s%s\n',startline,cmd);end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  if (verbose>2)
    S=sprintf('%s %% x is %s',startline,prop(x));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

function res=test2(verbose)
  if verbose>1, fprintf(' test 2 : constructor with 3 arguments\n'); end
  T=Thrust.types();
  C=Thrust.containers();
  fail=0;count=0;
    for type=T
      for container=C
%          whos
%            fprintf('%s %s\n',type{1},container{1});
        fail=fail+test2_simple(container{1},type{1},verbose);
        count=count+1;
      end
    end
  if verbose>1, fprintf(' test 2 : [%d/%d] OK\n',count-fail,count); end
  res=[fail,count];
end

function failure=test2_simple(container,type,verbose)  
  failure=0;
  dim=randi(10^5);
  startline='  ';
  %fprintf('%s %s\n',type,container);
  cmd=sprintf('x=mThrust(%d,''%s'',''%s'');',dim,container,type);
  
  if verbose>2,fprintf('%s%s\n',startline,cmd);end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  if (verbose>2)
    S=sprintf('%s %% x is %s',startline,prop(x));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

function res=test3(verbose)
  if verbose>1, fprintf(' test 3 : constructor with 2 arguments\n'); end
  T=Thrust.types('type','numerical');
  C=Thrust.containers();
  fail=0;count=0;
  for iscomplex={ false , true }
    for type=T
      for container=C
       %whos
  %        fprintf('%s %s',type{1},container{1});
        fail=fail+test3_simple(container{1},type{1},iscomplex{1},verbose);
        count=count+1;
      end
    end
  end
  if verbose>1, fprintf(' test 3 : [%d/%d] OK\n',count-fail,count); end
  res=[fail,count];
end

function failure=test3_simple(container,type,iscomplex,verbose)  
  failure=0;
  dim=randi(10^5);
  a=Thrust.valid.generate(dim,type,iscomplex);
%    if iscomplex
%      a=Thrust.toCtype(complex(1:dim,1:dim),type);
%    else
%      a=Thrust.toCtype(1:dim,type);
%    end
  startline='  ';
  cmd=sprintf('x=mThrust(a,''%s'');',container); %% a : %d-by-%d, %s',container,size(a,1),size(a,2),class(a));
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% a is %s\n',startline,arrayprop(a));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  if (verbose>2)
    S=sprintf('%s %% x is %s',startline,prop(x));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end