function varargout=or(varargin)
  p = inputParser; 
  p.addParamValue('verbose', 0, @(x) isscalar(x) & isnumeric(x));
  p.parse(varargin{:});
  verbose=p.Results.verbose;
  name=mfilename();
  if verbose>1, fprintf('TEST : "%s" function\n',name); end
  res=test1(verbose);
  Thrust.valid.tools.printFinalRes(name,res,verbose)
  if nargout==1, varargout{1}=res;end
end

function res=test1(verbose)
  if verbose>1, fprintf(' test 1 : Thrust<logical>= Thrust<logical> | Thrust<logical>\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for container=C
      fail=fail+test1_simple(container{1},verbose);
      count=count+1;
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 1 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test1_simple(container,verbose)  
  Conv=@(x) Thrust.toCtype(x,'logical');
  failure=0;
  dim=randi(10^5);
  x=Thrust.valid.generate(dim,'logical',false);
  y=Thrust.valid.generate(dim,'logical',false);
  gx=mThrust(x,container);
  gy=mThrust(y,container);
  startline='  ';
  cmd=sprintf('gz=(gx | gy); % gz=or(gx,gy);');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% gy is %s\n',startline,prop(gy));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  
  z=gather(gz);
  zz=(x|y);
  if ( sum(z==zz)~=dim ), failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gz is %s',startline,prop(gz));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

