function varargout=sequence(varargin)
  p = inputParser; 
  p.addParamValue('verbose', 0, @(x) isscalar(x) & isnumeric(x));
  p.parse(varargin{:});
  verbose=p.Results.verbose;
  name=sprintf('Thrust.%s',mfilename());
  if verbose>1, fprintf('TEST : "%s" function\n',name); end
  res=test1(verbose);
  Thrust.valid.tools.printFinalRes(name,res,verbose)
  if nargout==1, varargout{1}=res;end
end

function res=test1(verbose)
  if verbose>1, fprintf(' test 1 :\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test1_simple(container{1},type{1},verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 1 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test1_simple(container,type,verbose)  
  failure=0;
  dim=randi(10^4);
  init=randi(10^3);
  step=randi(10);
  startline='  ';
  cmd=sprintf('x=Thrust.sequence(%d,''container'',''%s'',''type'',''%s'',''init'',%d,''step'',%d);',dim,container,type,init,step);
  if verbose>2,fprintf('%s%s\n',startline,cmd);end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  X=Thrust.toCtype(init:step:(init+(dim-1)*step),type)';
  
  gx=gather(x);
  if sum(X-gx==0)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% x is %s',startline,prop(x));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end
  