function varargout=subsasgn(varargin)
  p = inputParser; 
  p.addParamValue('verbose', 0, @(x) isscalar(x) & isnumeric(x));
  p.parse(varargin{:});
  verbose=p.Results.verbose;
  name=mfilename();
  if verbose>1, fprintf('TEST : "%s" function\n',name); end
  res=test1(verbose);
  res=res+test2(verbose);
  res=res+test3(verbose);
  res=res+test4(verbose);
  res=res+test5(verbose);
  Thrust.valid.tools.printFinalRes(name,res,verbose)
  if nargout==1, varargout{1}=res;end
end

function res=test1(verbose)
  if verbose>1, fprintf(' test 1 : Thrust(array)=array\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types()
    for container=C
      fail=fail+test1_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test1_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 1 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test1_simple(container,type,iscomplex,verbose)  
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  I=unique(randi(dim,randi(dim),1));
  nI=length(I);
  x=Thrust.valid.generate(dim,type,iscomplex);
  y=Thrust.valid.generate(nI,type,iscomplex);
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('gx(I)=y;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% y is %s\n',startline,arrayprop(y));
    fprintf('%s %% I is %s\n',startline,arrayprop(I));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a','toto');
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gx);
  x(I)=y;
%        x.'
%        y.'
%        I.'
%        pause
  if sum(x==z)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gx is %s',startline,prop(gx));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%% TEST 2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test2(verbose)
  if verbose>1, fprintf(' test 2 : Thrust(Thrust<int>)=array\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types()
    for container=C
      fail=fail+test2_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test2_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 2 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test2_simple(container,type,iscomplex,verbose)  
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  I=unique(randi(dim,randi(dim),1));
  nI=length(I);
  x=Thrust.valid.generate(dim,type,iscomplex);
  y=Thrust.valid.generate(nI,type,iscomplex);
  gx=mThrust(x,container);
  gI=mThrust(Thrust.toCtype(I-1,'int'),container);
  startline='  ';
  cmd=sprintf('gx(gI)=y;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    %fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% y is %s\n',startline,arrayprop(y));
    fprintf('%s %% gI is %s\n',startline,prop(gI));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a',cmd);
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gx);
  x(I)=y;
%        x.'
%        y.'
%        I.'
%        pause
  if sum(x==z)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gx is %s',startline,prop(gx));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%% TEST 3 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test3(verbose)
  if verbose>1, fprintf(' test 3 : mThrust(:)=cst\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types()
    for container=C
      fail=fail+test3_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test3_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 3 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test3_simple(container,type,iscomplex,verbose)  
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  x=Thrust.valid.generate(dim,type,iscomplex);
  y=Thrust.valid.generate(1,type,iscomplex);
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('gx(:)=y;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% y is %s\n',startline,arrayprop(y));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a',cmd);
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gx);
  x(:)=y;
  if sum(x==z)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gx is %s',startline,prop(gx));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%% TEST 4 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test4(verbose)
  if verbose>1, fprintf(' test 4 : mThrust(:)=array\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types()
    for container=C
      fail=fail+test4_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test4_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 4 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test4_simple(container,type,iscomplex,verbose)  
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  x=Thrust.valid.generate(dim,type,iscomplex);
  y=Thrust.valid.generate(dim,type,iscomplex);
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('gx(:)=y;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% y is %s\n',startline,arrayprop(y));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a',cmd);
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gx);
  x(:)=y;
  if sum(x==z)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gx is %s',startline,prop(gx));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%% TEST 5 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test5(verbose)
  if verbose>1, fprintf(' test 5 : mThrust(:)=mThrust\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types()
    for container=C
      fail=fail+test5_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test5_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 5 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test5_simple(container,type,iscomplex,verbose)  
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  x=Thrust.valid.generate(dim,type,iscomplex);
  y=Thrust.valid.generate(dim,type,iscomplex);
  gx=mThrust(x,container);
  gy=mThrust(y,container);
  startline='  ';
  cmd=sprintf('gx(:)=gy;');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
   % fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% gy is %s\n',startline,prop(gy));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a',cmd);
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  z=gather(gx);
  if sum(y==z)<dim, failure=1;end
  if (verbose>2)
    S=sprintf('%s %% gx is %s',startline,prop(gx));
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end