function varargout=subsref(varargin)
  p = inputParser; 
  p.addParamValue('verbose', 0, @(x) isscalar(x) & isnumeric(x));
  p.parse(varargin{:});
  verbose=p.Results.verbose;
  name=mfilename();
  if verbose>1, fprintf('TEST : "%s" function\n',name); end
  res=test1(verbose);
  res=res+test2(verbose);
  Thrust.valid.tools.printFinalRes(name,res,verbose)
  if nargout==1, varargout{1}=res;end
end

function res=test1(verbose)
  if verbose>1, fprintf(' test 1 : Thrust=Thrust(array)\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types()
    for container=C
      fail=fail+test1_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test1_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 1 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test1_simple(container,type,iscomplex,verbose)  
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  x=Thrust.valid.generate(dim,type,iscomplex);
  I=randi(dim,randi(dim),1);
  gx=mThrust(x,container);
  startline='  ';
  cmd=sprintf('gy=gx(I);');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% I is %s\n',startline,arrayprop(I));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a',cmd);
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  y=gather(gy);
  if sum(x(I)==y)<length(y), failure=1;end
  if (verbose>2)
    if Thrust.ismThrust(gy)
      S=sprintf('%s %% gy is %s',startline,prop(gy));
    else
      S=sprintf('%s %% y is %s',startline,arrayprop(gy));
    end
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res=test2(verbose)
  if verbose>1, fprintf(' test 2 : Thrust=Thrust(Thrust<integer>)\n'); end
  C=Thrust.containers();
  fail=0;count=0;
  for type=Thrust.types()
    for container=C
      fail=fail+test2_simple(container{1},type{1},false,verbose);
      count=count+1;
    end
  end
  for type=Thrust.types('type','numerical')
    for container=C
      fail=fail+test2_simple(container{1},type{1},true,verbose);
      count=count+1;
    end
  end
  res=[fail,count];
  if verbose>1, fprintf(' test 2 : [%d/%d] OK\n',count-fail,count); end
end
  
function failure=test2_simple(container,type,iscomplex,verbose)  
  Conv=@(x) Thrust.toCtype(x,type);
  failure=0;
  dim=randi(10^5);
  x=Thrust.valid.generate(dim,type,iscomplex);
  I=Thrust.toCtype(randi(dim,randi(dim),1),'int');
  gx=mThrust(x,container);
  gI=mThrust(I-1,container); % take care with index with Thrust vector
  startline='  ';
  cmd=sprintf('gy=gx(gI);');
  if verbose>2
    fprintf('%s%s\n',startline,cmd);
    fprintf('%s %% gx is %s\n',startline,prop(gx));
    fprintf('%s %% gI is %s\n',startline,prop(gI));
  end
  try 
    eval(cmd);
  catch ME
    causeException = MException('MATLAB:mThrust:a',cmd);
    ME = addCause(ME,causeException);
    failure=1;
    rethrow(ME)
  end
  y=gather(gy);
  if sum(x(I)==y)<length(y), failure=1;end
  if (verbose>2)
    if Thrust.ismThrust(gy)
      S=sprintf('%s %% gy is %s',startline,prop(gy));
    else
      S=sprintf('%s %% y is %s',startline,arrayprop(gy));
    end
    line=repmat('.',1,70-length(S));
    fprintf('%s%s',S,line);
    if (failure==0), fprintf(' OK\n');else fprintf(' Failed\n');end
  end
end


