function debug(obj,varargin)
% debug Displays some information on an mThrust object
%    debug(A) or A.debug(), display information on the Thrust host_vector
%    or device_vector pointed by the mThrust object.
%
%    Optional key/value pairs :
%      - 'verbosity' key, the associated integer value must be is the degree
%        of verbosity. if value==0 (default), displays in green informations on 
%        Thrust host_vector or device_vector pointed by the mThrust object.
%        if value==1, displays moreover in red informations on the mThrust 
%        object.
%
%    Example
%       x = Thrust.sequence(10);
%       debug(x)
%       debug(x,'verbosity',1)
%
% Francois Cuvelier 2013-2016
  p = inputParser; 
  p.addParamValue('verbosity', 0, @(x) ismember(x,0:1));
  p.parse(varargin{:});
  R=p.Results;
  if R.verbosity>0
    varname=inputname(1);
    fprintf('\033[01;31m--- Matlab debugging class %s : varname = %s\n',class(obj),varname)
    fprintf('  - %s : \n',class(obj));
    fprintf('      handle    -> %.16X\n',obj.handle);
    fprintf('      dim       -> %d\n',obj.dim);
    fprintf('      container -> %s\n',obj.container);
    fprintf('      type      -> %s\n',obj.type);
    fprintf('      complex   -> %d\033[0m\n',obj.complex);
  end
  mex_mThrust_debug(obj);
  if R.verbosity>0
    fprintf('\033[01;31m--- End debugging : varname = %s\033[0m\n',varname)
  end
end