function obj=gt(obj1,obj2)
% > or GT  Greater than mThrust operator.
%    Z = (X > Y) or Z=gt(X,Y) denotes element-by-element comparison between 
%    X and Y. The i-th element of Z is set to 1 (true) if X(i)>Y(i) otherwise
%    is set to 0 (false).
%    X and Y can be 
%      - compatible mThrust vectors (same sizes, same types)
%      - X a mThrust vector and Y a scalar 
%      - X a scalar and Y a mThrust vector
%    The scalar values are converted to the type of the mThrust vector.
%
%    Example
%       x = Thrust.sequence(10);
%       y = Thrust.linspace(0,18,10);
%       z0 = x > y;
%       z1 = x > 3;
%       z2 = 2 > x;
%       z3 = gt(x,y);
%
% Francois Cuvelier 2013-2016
  obj=[];
  if ( Thrust.ismThrust(obj1) & Thrust.ismThrust(obj2) )
    assert(strcmp(obj1.type,obj2.type) & strcmp(obj1.container,obj2.container) & obj1.dim==obj2.dim, ...
           'mThrust:gt incompatible objects')
    obj=mThrust(obj1.dim,obj1.container,'logical');
    mex_mThrust_gt(obj1,obj2,obj);
    return
  end
  if ( Thrust.ismThrust(obj1) & isnumeric(obj2) )
    obj=mThrust(obj1.dim,obj1.container,'logical');
    mex_mThrust_gt(obj1,Thrust.toCtype(obj2,obj1.type),obj); 
    return
  end
  if ( Thrust.ismThrust(obj2) & isnumeric(obj1) )
    obj=mThrust(obj2.dim,obj2.container,'logical');
    mex_mThrust_lt(obj2,Thrust.toCtype(obj1,obj2.type),obj); 
    return
  end
end