function obj=ldivide(obj1,obj2)
% .\ or ldivide, mThrust vector left division.
%    Z = X.\Y or Z=ldivide(X,Y) returns element-by-element left division of 
%    X by Y. X and Y can be 
%      - compatible mThrust vectors (same sizes and same types)
%      - X a mThrust vector and Y a scalar 
%      - X a scalar and Y a mThrust vector
%    The scalar values are converted to the type of the mThrust vector.
%
%    Example
%       x = Thrust.sequence(10);
%       y = Thrust.linspace(10);
%       z0 = x.\y; % same as y./x
%       z1 = x.\3; % same as 3./x 
%       z2 = 2.\x; % same as x./2
%       z3 = ldivide(2,x);
%
% Francois Cuvelier 2013-2016
  if ( Thrust.ismThrust(obj1) && Thrust.ismThrust(obj2) ) % mThrust<type>  <-  mThrust<type> .\ mThrust<type>
    assert(obj1.dim==obj2.dim,'Dimension trouble');
    assert(strcmp(obj1.type,obj2.type),'Type trouble');
    assert(ismember(obj1.type,Thrust.types('type','real')));
    assert(strcmp(obj1.container,obj2.container),'Container trouble');
    obj=mThrust(obj1.dim,obj1.container,obj1.type,obj1.complex | obj2.complex);
    mex_mThrust_rdivide(obj2,obj1,obj);
  elseif ( (Thrust.ismThrust(obj1)) && ( isnumeric(obj2) ) ) % mThrust<type>  <-  mThrust<type> .\ scalar<type>
    assert(length(obj2)==1,'dimension failed');
    A=Thrust.toCtype(obj2,obj1.type);
    obj=mThrust(obj1.dim,obj1.container,obj1.type);
    mex_mThrust_mrdivide(obj1,A,obj);
  elseif ( (Thrust.ismThrust(obj2)) && ( isnumeric(obj1) ) ) % mThrust<type>  <-  scalar<type> .\ mThrust<type>
    assert(length(obj1)==1,'dimension failed');
    A=Thrust.toCtype(obj1,obj2.type);
    obj=mThrust(obj2.dim,obj2.container,obj2.type);
    mex_mThrust_mrdivide(obj2,A,obj);
  else
    error('incompatible type')
  end
end