function obj=mrdivide(obj1,obj2)
% *  mThrust vector division by a scalar.
%    Z = X/Y or Z=mrdivide(X,Y) denotes division of the mThrust vector X by 
%    the scalar Y. Y is converted to the type of the mThrust vector.
%
%    Example
%       x = Thrust.sequence(10);
%       z1 = x/3;
%       z2 = x/(1+i); % complex
%       z3 = mrdivide(x,(1-i)); % complex 
%
% Francois Cuvelier 2013-2016
  assert( Thrust.ismThrust(obj1) & isnumeric(obj2) )
  assert(length(obj2)==1,'dimension failed');
  if ~ismember(obj1.type,Thrust.types('type','real'))
    error('incompatible type')
  end
  A=Thrust.toCtype(obj2,obj1.type);
  isReal=(isreal(obj2) & isreal(A) );
  obj=mThrust(obj1.dim,obj1.container,obj1.type,~isReal);
  mex_mThrust_mrdivide(obj1,A,obj);
end