/*
*/
template <typename T,typename Vector,typename VectorOut> 
void mx_mThrust_op_complex(const mxArray *prhs[]){
  typedef typename Vector::value_type Tin;
  typedef typename VectorOut::value_type Tout;
  Vector &gx = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  VectorOut &gz = get_object<VectorOut>(mxGetProperty(prhs[2],0,"handle"));
  assert((gx.size()==gz.size()));
  T *pa_r = (T *)mxGetPr(prhs[1]);
  T *pa_i = (T *)mxGetPi(prhs[1]);
  thrust::complex<T> a(*pa_r,*pa_i);
  thrust::transform(gx.begin(),gx.end(),gz.begin(), op_functor<Tin,thrust::complex<T>,Tout>(a));
}

template <typename T,typename Vector,typename VectorOut> 
void mx_mThrust_op_real(const mxArray *prhs[]){
  typedef typename Vector::value_type Tin;
  typedef typename VectorOut::value_type Tout;
  Vector &gx = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  VectorOut &gz = get_object<VectorOut>(mxGetProperty(prhs[2],0,"handle"));
  assert((gx.size()==gz.size()));
  T *pa = (T *)mxGetPr(prhs[1]);  
  thrust::transform(gx.begin(),gx.end(),gz.begin(), op_functor<Tin,T,Tout>(*pa));
}

template  <typename T>
void mx_mThrust_opcst_type(const mxArray *prhs[] ){
  char *container[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"container")),
    mxArrayToString(mxGetProperty(prhs[2],0,"container"))
  };
  assert( (strcmp(container[0],container[1])==0) );
  bool iscomplex[3]; 
  logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[0],0,"complex"));
  iscomplex[0]=*iscomp;
  iscomplex[1]=mxIsComplex(prhs[1]);
  iscomp=(logical *)mxGetPr(mxGetProperty(prhs[2],0,"complex"));
  iscomplex[2]=*iscomp;
  if (strcmp(container[0], "host")==0){
    typedef thrust::host_vector<thrust::complex<T> > VecCplx;
    typedef thrust::host_vector<T> VecReal;
    if (iscomplex[0] && iscomplex[1])
      mx_mThrust_op_complex<T,VecCplx,VecCplx>(prhs);
    else if (iscomplex[0]) 
      mx_mThrust_op_real<T,VecCplx,VecCplx>(prhs);
    else if (iscomplex[1])
      mx_mThrust_op_complex<T,VecReal,VecCplx>(prhs);
    else
      mx_mThrust_op_real<T,VecReal,VecReal>(prhs);
  }else if (strcmp(container[0], "device")==0){
    typedef thrust::device_vector<thrust::complex<T> > VecCplx;
    typedef thrust::device_vector<T> VecReal;
    if (iscomplex[0] && iscomplex[1])
      mx_mThrust_op_complex<T,VecCplx,VecCplx>(prhs);
    else if (iscomplex[0]) 
      mx_mThrust_op_real<T,VecCplx,VecCplx>(prhs);
    else if (iscomplex[1])
      mx_mThrust_op_complex<T,VecReal,VecCplx>(prhs);
    else
      mx_mThrust_op_real<T,VecReal,VecReal>(prhs);
  }else
    mexErrMsgIdAndTxt(MESS,"Unknow container."); 
}

// binary arithmetic operator <OP> : +,-,* (for division / int and unsigned are not allowed)
// prhs[0] : input mThrust vector {Vin}
// prhs[1] : input matlab constant (real or complex) {a} 
// prhs[2] : output mThrust vector {vout <- Vin <OP> a
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 3) 
    mexErrMsgTxt("Bad input.");
  
  assert(mxIsClass(prhs[0],"mThrust") && mxIsClass(prhs[2],"mThrust") && mxIsScalar(prhs[1]));
  
  char *type[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"type")),
    mxArrayToString(mxGetProperty(prhs[2],0,"type"))
  };
  assert( strcmp(type[0],type[1])==0 );
  if (strcmp(type[0], "int")==0){
    assert( (mxGetClassID(prhs[1])==mxINT32_CLASS) );
    mx_mThrust_opcst_type<int>(prhs);
  }else if (strcmp(type[0], "unsigned")==0){
    assert( (mxGetClassID(prhs[1])==mxUINT32_CLASS) );
    mx_mThrust_opcst_type<unsigned>(prhs);
  }else if (strcmp(type[0], "float")==0){
    assert( (mxGetClassID(prhs[1])==mxSINGLE_CLASS) );
    mx_mThrust_opcst_type<float>(prhs);
  }else if (strcmp(type[0], "double")==0){
    assert( (mxGetClassID(prhs[1])==mxDOUBLE_CLASS) );
    mx_mThrust_opcst_type<double>(prhs);
  }else
    mexErrMsgIdAndTxt("mThrustToolbox:plus","type not implemented.");
  
  MEXPRINTF("ending");
}