/*
  for set operators : union, intersection, difference
  No complex values
*/

template <typename Vector> 
void mx_mThrust_op(const mxArray *prhs[]){
  Vector &gx = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  Vector &gy = get_object< Vector >(mxGetProperty(prhs[1],0,"handle"));
  Vector &gz = get_object< Vector >(mxGetProperty(prhs[2],0,"handle"));
  set_operator<Vector>(gx,gy,gz);
}


template  <typename T>
void mx_mThrust_op_type(const mxArray *prhs[]){
  char *container[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"container")),
    mxArrayToString(mxGetProperty(prhs[1],0,"container")), 
    mxArrayToString(mxGetProperty(prhs[2],0,"container"))
  };
  assert( (strcmp(container[0],container[1])==0) && (strcmp(container[0],container[2])==0) );
  bool iscomplex[3];
  for (int i=0;i<3;i++){
    logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[i],0,"complex"));
    iscomplex[i]=*iscomp;
  }
  assert( ~iscomplex[0] && ~iscomplex[1] && ~iscomplex[2] );
  
  if (strcmp(container[0], "host")==0){
    typedef thrust::host_vector<T> VecReal;
    mx_mThrust_op<VecReal>(prhs);  
  }else if (strcmp(container[0], "device")==0){
    typedef thrust::device_vector<T> VecReal;
    mx_mThrust_op<VecReal>(prhs);  
  }else{
    mexWarnMsgIdAndTxt(MESS,"Unknow container."); 
  }
  mxFree(container[0]);mxFree(container[1]);mxFree(container[2]);
}

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{ 
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 3) 
    mexErrMsgTxt("Bad input.");
  assert(mxIsClass(prhs[0],"mThrust"));
  assert(mxIsClass(prhs[1],"mThrust"));
  assert(mxIsClass(prhs[2],"mThrust"));
  
  char *type[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"type")),
    mxArrayToString(mxGetProperty(prhs[1],0,"type")), 
    mxArrayToString(mxGetProperty(prhs[2],0,"type"))
  };
  assert( (strcmp(type[0],type[1])==0) && (strcmp(type[0],type[2])==0) );
  
  if (strcmp(type[0], "int")==0){
    mx_mThrust_op_type<int>(prhs);
  }else if (strcmp(type[0], "unsigned")==0){
    mx_mThrust_op_type<unsigned>(prhs);
  }else if (strcmp(type[0], "float")==0){
    mx_mThrust_op_type<float>(prhs);
  }else if (strcmp(type[0], "double")==0){
    mx_mThrust_op_type<double>(prhs);
  }else{
    mxFree(type[0]);mxFree(type[1]);mxFree(type[2]);
    mexWarnMsgIdAndTxt(MESS,"type not implemented.");
  }
  mxFree(type[0]);mxFree(type[1]);mxFree(type[2]);
  MEXPRINTF("ending");
}