/*
*/

template <typename Vector> 
void mx_mThrust_fun_real(mxArray *plhs[],const mxArray *prhs[]){
  typedef typename Vector::value_type T;
  Vector &V = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  T *pa = (T *)mxGetPr(plhs[0]);  
  *pa=thrust::reduce(V.begin(),V.end(),START, fun_functor<T>());
}

template  <typename T>
void mx_mThrust_fun_type(mxArray *plhs[],const mxArray *prhs[], mxClassID classid){
  char *container = mxArrayToString(mxGetProperty(prhs[0],0,"container"));
  
  logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[0],0,"complex"));
  if (*iscomp)
    mexErrMsgIdAndTxt(MESS,"mThrust complex vector not allowed.");
  mwSize dims0[1]={1};
  plhs[0] = mxCreateNumericArray(1,dims0,classid,mxREAL );
  
  if (strcmp(container, "host")==0){
    typedef thrust::host_vector<T> VecReal;
    mx_mThrust_fun_real<VecReal>(plhs,prhs);
  }else if (strcmp(container, "device")==0){
    typedef thrust::device_vector<T> VecReal;
    mx_mThrust_fun_real<VecReal>(plhs,prhs);
  }else
    mexWarnMsgIdAndTxt(MESS,"Unknow container.");
  mxFree(container);
}

// for functions : sum, prod
// prhs[0] : input mThrust vector {Vin}
// plhs[0] : output matlab value (real or complex) {a} 
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=1 || nrhs != 1) 
    mexErrMsgTxt("Bad input.");
  if ( !mxIsClass(prhs[0],"mThrust") ){
    mexWarnMsgIdAndTxt(MESS,"Parameter must be a mThrust vector");
    return;
  }
  
  char *type = mxArrayToString(mxGetProperty(prhs[0],0,"type"));
  
  if (strcmp(type, "double")==0){
    mx_mThrust_fun_type<double>(plhs,prhs,mxDOUBLE_CLASS);
  }else if (strcmp(type, "float")==0){
    mx_mThrust_fun_type<float>(plhs,prhs,mxSINGLE_CLASS);
  }else if (strcmp(type, "int")==0){
    mx_mThrust_fun_type<int>(plhs,prhs,mxINT32_CLASS);
  }else if (strcmp(type, "unsigned")==0){
    mx_mThrust_fun_type<unsigned>(plhs,prhs,mxUINT32_CLASS);
  }else
    mexWarnMsgIdAndTxt(MESS,"type <%s> not implemented.",type);
  mxFree(type);
  MEXPRINTF("ending");
}