#include <iostream>
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"
#include <thrust/copy.h>

#include <thrust/set_operations.h>

#define MESS "mThrustToolbox:find"

template <typename T>
struct is_nonzeros : public thrust::unary_function<T,bool>
{
  __host__ __device__
  bool operator()(T x)
  {
    return (x != (T)0);
  }
};

template <typename Vector,typename VectorInt>
void mx_mThrust_find(const mxArray *prhs[])
{
  typedef typename Vector::value_type  T;
  typename VectorInt::iterator I_end;
  Vector  &A = get_object<Vector >(mxGetProperty(prhs[0],0,"handle"));
  VectorInt &I = get_object<VectorInt>(mxGetProperty(prhs[1],0,"handle"));
  I_end = thrust::copy_if(thrust::make_counting_iterator<int>(0),
                   thrust::make_counting_iterator<int>(A.size()), // indices from 0 to N
                   A.begin(),                             // array data
                   I.begin(),                    // result will be written here
                   is_nonzeros<T>());
  // shrink I to exactly fit output
  I.erase(I_end, I.end());
}

template  <typename T>
void mx_mThrust_find_type(const mxArray *prhs[]){
  char *container[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"container")),
    mxArrayToString(mxGetProperty(prhs[1],0,"container"))
  };
  if ( strcmp(container[0],container[1])!=0 ){
    mexWarnMsgIdAndTxt(MESS,"The mThrust vectors must have the same container.");
    return;
  }
  bool iscomplex[2];
  for (int i=0;i<2;i++){
    logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[i],0,"complex"));
    iscomplex[i]=*iscomp;
  }
  if ( iscomplex[1] ){
    mexWarnMsgIdAndTxt(MESS,"2nd parameter can't a mThrust complex vector.");
    return;
  }
  if (strcmp(container[0], "device")==0){
    typedef thrust::device_vector<thrust::complex<T> > VecCplx;
    typedef thrust::device_vector<T> VecReal;
    typedef thrust::device_vector<int> VecInt;
    if (iscomplex[0])
      mx_mThrust_find<VecCplx,VecInt>(prhs); 
    else
      mx_mThrust_find<VecReal,VecInt>(prhs);
  }else if (strcmp(container[0], "host")==0){
    typedef thrust::host_vector<thrust::complex<T> > VecCplx;
    typedef thrust::host_vector<T> VecReal;
    typedef thrust::host_vector<int> VecInt;
    if (iscomplex[0])
      mx_mThrust_find<VecCplx,VecInt>(prhs); 
    else
      mx_mThrust_find<VecReal,VecInt>(prhs);
  }else 
    mexWarnMsgIdAndTxt(MESS,"Unknow container %s.",container[0]); 
  mxFree(container[0]);mxFree(container[1]);
}

void mx_mThrust_find_logical(const mxArray *prhs[]){
  char *container[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"container")),
    mxArrayToString(mxGetProperty(prhs[1],0,"container"))
  };
  if ( strcmp(container[0],container[1])!=0 ){
    mexWarnMsgIdAndTxt(MESS,"The mThrust vectors must have the same container.");
    return;
  }
  
  if (strcmp(container[0], "device")==0){
    mx_mThrust_find<thrust::device_vector<logical>,thrust::device_vector<int> >(prhs); 
  }else if (strcmp(container[0], "host")==0){
    mx_mThrust_find<thrust::host_vector<logical>,thrust::host_vector<int> >(prhs); 
  }else 
    mexWarnMsgIdAndTxt(MESS,"Unknow container %s.",container[0]); 
  mxFree(container[0]);mxFree(container[1]);
}

// prhs[0] mThrust object to find zeros
// prhs[1] mThrust <int> object (same container as prhs[0])
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=0 || nrhs != 2) 
    mexErrMsgTxt("Bad input... To do");
  
  assert(mxIsClass(prhs[0],"mThrust") && mxIsClass(prhs[1],"mThrust") );
  char *type[] = {
    mxArrayToString(mxGetProperty(prhs[0],0,"type")),
    mxArrayToString(mxGetProperty(prhs[1],0,"type"))
  };
  if ( strcmp(type[1],"int")!=0 ){
    mexWarnMsgIdAndTxt(MESS,"2nd parameter must be a mThrust 'int' vector.");
    mxFree(type[0]);mxFree(type[1]);
    return;
  }
  
  if (strcmp(type[0], "double")==0)
    mx_mThrust_find_type<double>(prhs);
  else if (strcmp(type[0], "float")==0)
    mx_mThrust_find_type<float>(prhs);
  else if (strcmp(type[0], "int")==0)
    mx_mThrust_find_type<int>(prhs);
  else if (strcmp(type[0], "unsigned")==0)
    mx_mThrust_find_type<unsigned>(prhs);
  else if (strcmp(type[0], "logical")==0)
    mx_mThrust_find_logical(prhs);
  else
    mexWarnMsgIdAndTxt(MESS,"Find of mThrust<%s> not allowed.",type);
  mxFree(type[0]);mxFree(type[1]);
}