#include <iostream>
#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"

template <typename Vector> void mx_mThrust_getSize(mxArray *plhs[],const mxArray *prhs[]){
  Vector &V = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  plhs[0]=mxCreateDoubleScalar((double) V.size());  
}


template  <typename T>
void mx_mThrust_getSize_type(mxArray *plhs[],const mxArray *prhs[]){
  char *container = mxArrayToString(mxGetProperty(prhs[0],0,"container"));
  
  logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[0],0,"complex"));
  
  if (strcmp(container, "device")==0){
    if (*iscomp){
      mx_mThrust_getSize<thrust::device_vector<thrust::complex<T> > >(plhs,prhs);
    }else 
      mx_mThrust_getSize<thrust::device_vector<T> >(plhs,prhs); 
  }else if (strcmp(container, "host")==0){
    if (*iscomp)
      mx_mThrust_getSize<thrust::host_vector<thrust::complex<T> > >(plhs,prhs);
    else 
      mx_mThrust_getSize<thrust::host_vector<T> >(plhs,prhs);  
  }else
    mexWarnMsgIdAndTxt("mThrustToolbox:getSize","Unknow container."); 
  mxFree(container);
}

// prhs[0] : handle to input mThrust object
// plhs[0] : size of the Thrust vector
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=1 || nrhs != 1) 
    mexErrMsgTxt("Bad input/output... To do");
  
  assert( mxIsClass(prhs[0],"mThrust") );
  
  char *type = mxArrayToString(mxGetProperty(prhs[0],0,"type"));
  
  if (strcmp(type, "double")==0)
    mx_mThrust_getSize_type<double>(plhs,prhs);
  else if (strcmp(type, "float")==0)
    mx_mThrust_getSize_type<float>(plhs,prhs);
  else if (strcmp(type, "int")==0)
    mx_mThrust_getSize_type<int>(plhs,prhs);
  else if (strcmp(type, "unsigned")==0)
    mx_mThrust_getSize_type<unsigned>(plhs,prhs);
  else if (strcmp(type, "logical")==0)
    mx_mThrust_getSize_type<logical>(plhs,prhs);
  else{
    mexWarnMsgIdAndTxt("mThrustToolbox:getSize","type not implemented.");
  }
  mxFree(type);
  MEXPRINTF("ending");
}