#include <string>
#include "textcolors.h"
#include "ObjectHandle.h"
#include "mThrust.h"
#include "mex.h"
#include <thrust/reduce.h>
#include <thrust/functional.h>

#define MESS "mThrustToolbox:sum"
#define START 0

template <typename T> struct fun_functor :  public thrust::binary_function<T,T,T>
{
  __host__ __device__
  T operator()(const T & z1, const T& z2) const { return z1+z2; }
};

//struct thrust::multiplies<T> fun_functor;
struct logical_functor :  public thrust::binary_function<logical,logical,int>
{
  __host__ __device__
  int operator()(const logical & z1, const logical& z2) const { 
    int a=(z1?1:0);
    int b=(z2?1:0);
    return a+b;
    //return (z1?int(1):int(0))+(z2?int(1):int(0)); 
  }
};
//#include "mex_mThrust_cmathfun11_gen.cu"

/*
 */

template <typename T,typename Vector> 
void mx_mThrust_fun_complex(mxArray *plhs[],const mxArray *prhs[], mxClassID classid){
  Vector &V = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  mwSize dims0[1]={1};
  plhs[0] = mxCreateNumericArray(1,dims0,classid, mxCOMPLEX );
  T *pa_r = (T *)mxGetPr(plhs[0]);
  T *pa_i = (T *)mxGetPi(plhs[0]);
  thrust::complex<T> a,start(START,0);
  a=thrust::reduce(V.begin(),V.end(),start, fun_functor<thrust::complex<T> >() );
  *pa_r=a.real();*pa_i=a.imag();
}

template <typename Vector> 
void mx_mThrust_fun_real(mxArray *plhs[],const mxArray *prhs[], mxClassID classid){
  typedef typename Vector::value_type T;
  Vector &V = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  mwSize dims0[1]={1};
  plhs[0] = mxCreateNumericArray(1,dims0,classid, mxREAL );
  T *pa = (T *)mxGetPr(plhs[0]);  
  *pa=(T)thrust::reduce(V.begin(),V.end());
}

template <typename Vector,typename VectorInt> 
void mx_mThrust_fun_logical(mxArray *plhs[],const mxArray *prhs[]){
  Vector &V = get_object< Vector >(mxGetProperty(prhs[0],0,"handle"));
  VectorInt Vint(V.begin(),V.end()); // reduce seem not to work with logical
  int *pa = (int *)mxGetPr(plhs[0]);  
  *pa=thrust::reduce(Vint.begin(),Vint.end());//,START, logical_functor());
}

template  <typename T>
void mx_mThrust_fun_type(mxArray *plhs[],const mxArray *prhs[], mxClassID classid){
  char *container = mxArrayToString(mxGetProperty(prhs[0],0,"container"));
  logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[0],0,"complex"));
  
  if (strcmp(container, "host")==0){
    typedef thrust::host_vector<thrust::complex<T> > VecCplx;
    typedef thrust::host_vector<T> VecReal;
    if (*iscomp)
      mx_mThrust_fun_complex<T,VecCplx>(plhs,prhs,classid);
    else 
      mx_mThrust_fun_real<VecReal>(plhs,prhs,classid);
  }else if (strcmp(container, "device")==0){
    typedef thrust::device_vector<thrust::complex<T> > VecCplx;
    typedef thrust::device_vector<T> VecReal;
    if (*iscomp)
      mx_mThrust_fun_complex<T,VecCplx>(plhs,prhs,classid);
    else 
      mx_mThrust_fun_real<VecReal>(plhs,prhs,classid);
  }else
    mexWarnMsgIdAndTxt(MESS,"Unknow container.");
  mxFree(container);
}

void mx_mThrust_fun_logical(mxArray *plhs[],const mxArray *prhs[]){
  char *container = mxArrayToString(mxGetProperty(prhs[0],0,"container"));
  
  // logical con't be complex
  // logical *iscomp=(logical *)mxGetPr(mxGetProperty(prhs[0],0,"complex"));
  mwSize dims0[1]={1};
  plhs[0] = mxCreateNumericArray(1,dims0,mxINT32_CLASS,mxREAL);
  
  if (strcmp(container, "host")==0){
    typedef thrust::host_vector<logical> VecReal;
    typedef thrust::host_vector<int> VecInt;
    mx_mThrust_fun_logical<VecReal,VecInt>(plhs,prhs);
  }else if (strcmp(container, "device")==0){
    typedef thrust::device_vector<logical> VecReal;
    typedef thrust::device_vector<int> VecInt;
    mx_mThrust_fun_logical<VecReal,VecInt>(plhs,prhs);
  }else
    mexWarnMsgIdAndTxt(MESS,"Unknow container.");
  mxFree(container);
}

// for functions : sum, prod
// prhs[0] : input mThrust vector {Vin}
// plhs[0] : output matlab value (real or complex) {a} 
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  MEXPRINTF("starting");
  if (nlhs !=1 || nrhs != 1) 
    mexErrMsgTxt("Bad input.");
  if ( !mxIsClass(prhs[0],"mThrust") ){
    mexWarnMsgIdAndTxt(MESS,"Parameter must be a mThrust vector");
    return;
  }
  
  char *type = mxArrayToString(mxGetProperty(prhs[0],0,"type"));
  
  if (strcmp(type, "double")==0){
    mx_mThrust_fun_type<double>(plhs,prhs,mxDOUBLE_CLASS);
  }else if (strcmp(type, "float")==0){
    mx_mThrust_fun_type<float>(plhs,prhs,mxSINGLE_CLASS);
  }else if (strcmp(type, "int")==0){
    mx_mThrust_fun_type<int>(plhs,prhs,mxINT32_CLASS);
  }else if (strcmp(type, "unsigned")==0){
    mx_mThrust_fun_type<unsigned>(plhs,prhs,mxUINT32_CLASS);
  }else if (strcmp(type, "logical")==0){
    mx_mThrust_fun_logical(plhs,prhs);
  }else
    mexWarnMsgIdAndTxt(MESS,"type not implemented.");
  mxFree(type);
  MEXPRINTF("ending");
}